#version 130


////////////////////////////////////////////////////ADJUSTABLE VARIABLES/////////////////////////////////////////////////////////
#define POM 								//Comment to disable parallax occlusion mapping.
#define NORMAL_MAP_MAX_ANGLE 0.5f   		//The higher the value, the more extreme per-pixel normal mapping (bump mapping) will be.
///////////////////////////////////////////////////END OF ADJUSTABLE VARIABLES///////////////////////////////////////////////////



uniform sampler2D texture;
uniform sampler2D lightmap;
uniform sampler2D normals;
uniform sampler2D specular;
//uniform float rainStrength;
uniform float wetness;


varying vec4 color;
varying vec4 texcoord;
varying vec4 lmcoord;

varying vec3 viewVector;
varying vec3 normal;
varying vec3 tangent;
varying vec3 binormal;
varying vec3 globalNormal;

varying float materialIDs;

varying float distance;

const float MAX_OCCLUSION_DISTANCE = 100.0;

const int MAX_OCCLUSION_POINTS = 20;

const int GL_LINEAR = 9729;
const int GL_EXP = 2048;

const float bump_distance = 78.0f;
const float fademult = 0.1f;





/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////ALEXEI03A_BEGIN////////////////////////////////////////////////////////////

#define X 16 //DO NOT TOUCH
#define Y 16 //DO NOT TOUCH

const float pom_depth = 0.1;
const float pom_distance = 10.0f;
const int linear_steps = 60;
const int binary_steps = 16;

vec2 get_one(){
    return fract(texcoord.st * vec2(X, Y));
}

vec4 texture_one(sampler2D tex, vec2 coord){
    vec2 tile = vec2(X, Y);
    vec2 root = texcoord.st - mod(texcoord.st, 1/tile);
    vec2 smpl = fract(coord) / tile;
    return texture2D(tex, root + smpl);
}

vec4 texture_linear(sampler2D tex, vec2 coord){
    vec2 uv = fract(coord);
    vec2 tile = vec2(X, Y);
    vec2 textureSize = textureSize(tex, 0) / tile;
    vec2 texelSize = 1 / textureSize;
    vec2 f = fract(uv.xy * textureSize);
    vec4 tl = texture_one(tex, uv);
    vec4 tr = texture_one(tex, uv + vec2(texelSize.s, 0));
    vec4 bl = texture_one(tex, uv + vec2(0, texelSize.t));
    vec4 br = texture_one(tex, uv + vec2(texelSize.s, texelSize.t));
    vec4 tA = mix( tl, tr, f.x );
    vec4 tB = mix( bl, br, f.x );
    return mix( tA, tB, f.y );
}

float find_intersection(vec2 dp, vec2 ds) {
    float depth_step = 1.0 / linear_steps;
    float size = depth_step;
    float depth = 1.0;
    float best_depth = 1.0;
    for (int i = 0 ; i < linear_steps - 1 ; ++i) {
        depth -= size;
        vec4 t = texture_linear(normals, fract(dp + ds * depth));
        if (depth >= 1.0 - t.a) best_depth = depth;
    }
    depth = best_depth - size;
    for (int i = 0 ; i < binary_steps ; ++i) {
        size *= 0.5;
        vec4 t = texture_linear(normals, fract(dp + ds * depth));
        if (depth >= 1.0 - t.a) {
            best_depth = depth;
            depth -= 2 * size;
        }
        depth += size;
    }
    return best_depth;
}

vec2 calculate_pom_one(){
    mat3 tbnMatrix = transpose(mat3(tangent.xyz, binormal.xyz, normal.xyz));
    vec2 dp = get_one();
    if(distance <= pom_distance && texture_linear(normals, dp).a < 1.0f && normalize(tbnMatrix * viewVector).z <= 0.0){
        vec3 eview = normalize(viewVector.xyz);
        vec3 tview = normalize(vec3(dot(eview, tangent), dot(eview, binormal), dot(eview, -normal)));
        vec2 ds = tview.xy * pom_depth / tview.z;
        float dist = find_intersection(dp, ds);
        return fract(dp + dist * ds);
    } else {
        return dp;
    }
}

//////////////////////////////////////////////////////ALEXEI03A_END//////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



void main() {	

	//store lightmap in auxilliary texture. r = torch light. g = lightning. b = sky light.
		//vec4 lightmap = texture2D(lightmap, lmcoord.st);
		vec4 lightmap = vec4(0.0f, 0.0f, 0.0f, 1.0f);
		
		//Separate lightmap types
		lightmap.r = clamp((lmcoord.s * 33.05f / 32.0f) - 1.05f / 32.0f, 0.0f, 1.0f);
		lightmap.b = clamp((lmcoord.t * 33.05f / 32.0f) - 1.05f / 32.0f, 0.0f, 1.0f);

	
	#ifdef POM
		vec2 adjustedTexCoord = calculate_pom_one(); //texcoord.st;
	#else
		vec2 adjustedTexCoord = get_one();
	#endif
	float texinterval = 0.0625f;
	
	vec3 coord = vec3(texcoord.st, 1.0);

	
	
	//Diffuse
		vec4 albedo = texture_linear(texture, adjustedTexCoord) * color;
		//Wet diffuse
		const float wetPow = 1.75f;
		
		float metallicMask = 0.0f;
		
		if (   abs(materialIDs - 20.0f) < 0.1f
			|| abs(materialIDs - 21.0f) < 0.1f
			|| abs(materialIDs - 22.0f) < 0.1f
			|| abs(materialIDs - 23.0f) < 0.1f) {
			metallicMask = 1.0f;
		}
		
		float dampMask = clamp(lightmap.b - 0.8f, 0.0f, 0.2f) / 0.2f;
			  dampMask *= wetness;
			  dampMask *= mix(1.0f, 0.0f, metallicMask);
			  
		albedo.rgb = pow(albedo.rgb, vec3(mix(1.0f, wetPow, dampMask)));
		albedo.rgb *= mix(1.0f, 0.85f, dampMask);
		
		gl_FragData[0] = albedo;
		


		
	
	//Depth  
	float pomdepth = texture_linear(normals, adjustedTexCoord).a;

	float pomdepthbias = (1.0f - pomdepth) * (1.0f - gl_FragCoord.z) * (1.0f - gl_FragCoord.z);
	
	gl_FragData[1] = vec4(gl_FragCoord.z, lightmap.r, lightmap.b, 1.0f);
	
	vec4 spec = texture_linear(specular, adjustedTexCoord.st);
	
	
	float wetfactor = clamp(lightmap.b - 0.9f, 0.0f, 0.1f) / 0.1f;
	
	
	vec4 frag2;
	vec4 frag3;
	
	if (distance < bump_distance) {
	
			vec3 bump = texture_linear(normals, adjustedTexCoord).rgb * 2.0f - 1.0f;
			vec3 bump2 = texture_linear(normals, adjustedTexCoord).rgb * 2.0f - 1.0f;
				 bump2.g = 1.0f - bump2.g;
			
			float bumpmult = clamp(bump_distance * fademult - distance * fademult, 0.0f, 1.0f) * NORMAL_MAP_MAX_ANGLE;
	              bumpmult *= 1.0f - (spec.g * 0.9f * wetness * wetfactor);
				  
			bump = bump * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult);
			
			
			bump2 = bump2 * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult);
		
			mat3 tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
								  tangent.y, binormal.y, normal.y,
						     	  tangent.z, binormal.z, normal.z);
			
			frag2 = vec4(bump * tbnMatrix * 0.5 + 0.5, 1.0);
			
			frag3 = vec4((globalNormal + (bump2 * NORMAL_MAP_MAX_ANGLE)) * 0.5f + 0.5f, 1.0f);
			
	} else {
	
			frag2 = vec4((normal) * 0.5f + 0.5f, 1.0f);		
			frag3 = vec4(globalNormal * 0.5f + 0.5f, 1.0f);
			
	}

	
	

	
	//Material ID's
		
		//Combine material masks
		float mats_1 = materialIDs;

	
	gl_FragData[2] = frag2;
		
	//matIDs, specularity, iswater
	gl_FragData[3] = vec4(mats_1/255.0f, spec.r + spec.b + spec.g * wetness * wetfactor, 0.0f, 1.0f);	
	
	//specularity, blank, iswater
	//gl_FragData[3] = vec4(spec.r + spec.b + spec.g * wetness, 0.0f, 0.0f, 1.0f);
	
	//gaux3
	//gl_FragData[6] = vec4(0.0f, 0.0f, 0.0f, 1.0f);

	

}